function scheme07h(nu, IniCon, Tfinal, Npoint)
% Crank-Nicolson Finite Differences for u_t = u_xx.
% ____________________________________________________________________________
% 18.311 (Principles of Applied Mathematics). MIT, Spring 2013. R. R. Rosales.
%
% function scheme07a(nu, IniCon, Tfinal, Npoint)
%
% Finite Differences Numerical Scheme (demo).
% Equation. ...................................................... u_t = u_xx.
% Notation.  ........... u(n, m) = solution at x = x_n = n*dx, t = t_m = m*dt.
% Scheme name. ............................ Crank-Nicolson Finite Differences.
% Scheme parameters. ................................................... NONE.
% Numerical parameter .......................................... nu = dt/dx^2.
% Scheme equation (implicit)
%
%     (1 + nu)*u(n, m+1) - 0.5*nu*(u(n+1, m+1) + u(n-1, m+1)) =
%     (1 - nu)*u(n, m  ) + 0.5*nu*(u(n+1, m  ) + u(n-1, m  )).
%
% For the meaning of the other variables,  and how to use the scheme, read the
% the file schemesREADME.m
%
% ============================================================================
Freeze = 250;
Length = 2*pi;
%
ninput = 4;% --- number of inputs.
if nargin < ninput;   Npoint = 30:30:120; end;
if nargin < ninput-1; Tfinal = 5.00;      end; 
%
Test   = -1;% On: Test = +1;   Off: Test = -1;
EQUATN = 'u_t = u_xx';         %@
SCHEME = 'Crank-Nicolson FD';  %@
NU     = 'dt/dx^2';            %@
EQNtex = 'u_t = u_{xx}';       %@
SCHtex = 'Crank-Nicolson FD';  %@
%
% ============================================================================
% ------------------------- MEANING OF THE VARIABLES -------------------------
%
% nu     = dt/dx^2;
% ........ NO parameters for this scheme.
% IniCon = selects the initial conditions.
% Tfinal = time for integration:         0 <= t <= Tfinal.
% Npoint = positive integer array with the number of points in the space grid.
%          Integrations are done with dx=Length/Npoint(n); n=1:length(Npoint).
% Freeze = cut off on solution's growth. If the errors grow, when the solution
%          reaches the size specified by Freeze, any further growth is stopped
%          by scaling down to size Freeze at each time step.  This is to avoid
%          problems with plotting, e.g.: axis limits over-flows.
% Length = period in space for solution: 0 <= x <= Length.
% Test   = Used to turn on/off (Test=1/-1) a test of the linear solver for the
%          scheme explicit equations --- compare them with an FFT solution.
%          ** USE FOR DEBUGGING ONLY. ***
%
% ---- STRINGS WITH TEXT TO BE USED  BY COMMAND LINE MESSAGES TO THE USER ----
%
% EQUATN = Equation solved.
% SCHEME = Name of the scheme used.
% NU     = Meaning of nu.
%
% ----------- STRINGS WITH LATEX TO BE USED FOR TITLES AND FIGURES -----------
%
% EQNtex = Equation solved.
% SCHtex = Name of the scheme used.
%
% ----------- BELOW: %@ marks places where different schemes could differ.
%
% ============================================================================
% Command line messages generated by demo, stating what it does.
%
fprintf( '\n Finite Differences Numerical Scheme (demo).')
fprintf(['\n EQUATION: ', EQUATN, '.'])
fprintf(['\n SCHEME:   ', SCHEME, '.'])
fprintf('\n Please read the file schemesREADME.')
fprintf('\n Rotate 3D will be on in the figures.')
fprintf('\n Type "close all" to close the figures.\n\n')
%
% ============================================================================
% Request the user to input parameters if not in the script call.           %@
%
NAR = 0;
NAR = NAR+1; if nargin < NAR
   nu = input(['   Enter the value of nu = ', NU, ' = ']);
   if isempty(nu); nu = 1; end
   fprintf('\n')
end;   if nu <= 0; nu = 1; end
%
% Request parameters here, if any.
%
NAR = NAR+1; if nargin < NAR
   IniCon = input('   Enter an integer to select the initial conditions = ');
   if isempty(IniCon); IniCon = 1; end
   fprintf('\n')
end;
IniCon = round(IniCon); if IniCon <= 0; IniCon = 1; end
%
% ============================================================================
% Define initial conditions (stored in u0) and integrate.
% Adding new options to the initial conditions is very simple.
%
for n = 1:length(Npoint)
   N  = Npoint(n);  dx = Length/N;  x = Length*(1:N)/N;  dt = nu*dx^2; %@
   u0 = schemesIniCon(IniCon, x, N);
   %
   U  = u0;% --- U will have the solution, one time level per row.
   T  =  0;% --- T will have the times computed.
   t  = dt;% --- t will have the time to be computed next.
   up = u0;% --- In each time step, up contains the solution at the current
   un = u0;%     time, and the solution at the next time goes into un.
   er =  0;% --- This is used by the linear solver testing step below.
   %
   while t <= Tfinal + dt
      %
      % SCHEME IMPLEMENTATION HERE. -------------------------------- %
      %
      % The equation for un is
      % [SCH]                   (1 + nu)*un - 0.5*nu*(Sp*un + Sm*un)
      %                       = (1 - nu)*up + 0.5*nu*(Sp*up + Sm*up).%@
      % where (Sp*v)(n) = v(n+1)
      % and   (Sm*v)(n) = v(n-1), for v periodic.
      %
      % Programming notes: the scheme equations are implicit. The
      % usual linear algebra ways to solve it would lead to a way
      % of writing the code which, as far as I can see, would not
      % have a form that would provide one-line-changes  from one
      % scheme to another.  The way below uses FFT and the growth
      % factor, which I have from the vNSA problems.   It may not
      % be the most obvious way to solve the scheme equations ...
      % but gives a very compact solution.    Thus, we use FFT's:
      %
      kappa = 2*pi*(0:N-1)/N;
      G  = (1-2*nu*(sin(kappa/2)).^2)./(1+2*nu*(sin(kappa/2)).^2);%@
      un = real(ifft(G.*fft(up)));
      %
      % --- TEST the linear solver. -------------------------------- %
      %
      if Test > 0
        Spup = [up(2:N), up(1)];   Smup = [up(N), up(1:N-1)];
        Spun = [un(2:N), un(1)];   Smun = [un(N), un(1:N-1)];
        wp   = (1 - nu)*up + 0.5*nu*(Spup + Smup);%@
        wn   = (1 + nu)*un - 0.5*nu*(Spun + Smun);%@
        er = max(max(abs(wn-wp)), er);
      end
      %
      % --- FREEZE ERROR GROWTH ONCE ERRORS GET LARGE ENOUGH. ------ %
      %     We do this by re-scaling solution to maximum allowed size.
      %
      uM = max(abs(un));   if uM > Freeze; un = (Freeze/uM)*un; end
      %
      % --- STORE UPDATED SOLUTION AT NEW TIME. -------------------- %
      %
      U = [U; un];   T = [T; t];   up = un;   t = t + dt;
   end
   if Test > 0; fprintf([' Error in solving linear equations <= ', ...
                         num2str(er), '\n']); end
   %
   % --- PLOT. ----------------------------------------------------- %
   %
   zmax = max(max(U));   zmin = min(min(U));
   MakeTheSchemeFigure(n, N);   whitebg('k');
   mesh([0, x], T, [U(:, N), U]);
   hold on;   grid on;   rotate3d on
   axis([0 Length 0 (t-dt) zmin zmax])
   %
   % --- Abreviations used for labels and similar. ----------------- %
   %
   LW  = 'LineWidth';    Pos = 'Position';              UN = 'Units';
   FW  = 'FontWeight';   HoA = 'HorizontalAlignment';   Lf = 'Left';
   FS  = 'FontSize';     Nor = 'Normalized';            
   FU  = 'FontUnits';    Str = 'String';
   Bf  = 'Bold';
   %
   % --- Pretty-up the plot. --------------------------------------- %
   %
   PSN = [0.1500 0.1500 0.7350 0.7050];
   set(gca, LW, 2, Pos, PSN, FW, Bf, FS, 16)
   colormap(hsv);  view(-16, 35); %@
   %
   St1 = NU; %@
   St2 = [' = ', num2str(nu, '%5.3f')]; %@
   St3 = 'dx';   St4 = ' = 2 \pi / N';
   St5 = 'N';    St6 = [' = ', num2str(N)];
   % =========================================================================
   text(Str,St1, UN,Nor, Pos,[0.780 1.140], HoA,Lf, FW,Bf, FU,Nor, FS,0.05)
   text(Str,St2, UN,Nor, Pos,[0.930 1.140], HoA,Lf, FW,Bf, FU,Nor, FS,0.05)
   text(Str,St3, UN,Nor, Pos,[0.780 1.080], HoA,Lf, FW,Bf, FU,Nor, FS,0.05)
   text(Str,St4, UN,Nor, Pos,[0.930 1.080], HoA,Lf, FW,Bf, FU,Nor, FS,0.05)
   text(Str,St5, UN,Nor, Pos,[0.780 1.020], HoA,Lf, FW,Bf, FU,Nor, FS,0.05)
   text(Str,St6, UN,Nor, Pos,[0.930 1.020], HoA,Lf, FW,Bf, FU,Nor, FS,0.05)
   % =========================================================================
   St7 = SCHtex;
   St8 = EQNtex;
%  St9 = ['\delta = ', num2str(delta, '%5.3f')]; %@  NO PARAMETERS HERE.
   % =========================================================================
   text(Str,St7, UN,Nor, Pos,[-0.15 1.140], HoA,Lf, FW,Bf, FU,Nor, FS,0.06)
   text(Str,St8, UN,Nor, Pos,[-0.15 1.050], HoA,Lf, FW,Bf, FU,Nor, FS,0.06)
%  text(Str,St9, UN,Nor, Pos,[ 0.16 1.073], HoA,Lf, FW,Bf, FU,Nor, FS,0.05)%@
   % =========================================================================
   xlabel('{\bf  x}', FU, Nor, FS, 0.10)
   ylabel('{\bf  t}', FU, Nor, FS, 0.10)
   zlabel('{\bf u }', FU, Nor, FS, 0.10, HoA, 'Right', 'Rotation', 0)
end
% ============================================================================
% Make plot of the computed solution at time t = Tfinal
% when there is only one run [i.e.: Npoint is not an array].
%
NOF = ['Solution with N = ', num2str(N), ' points in space. 3-D plot.'];
set(gcf, 'Name', NOF)
%
if n == 1
  MakeTheSchemeFigure(3, N);   whitebg('w');
  NOF = ['Solution with N = ', num2str(N), ' points in space,'];
  NOF = [NOF, ' at time t = ', num2str(Tfinal), '.'];
  set(gcf, 'Name', NOF)
  NOF = ['N = ', num2str(N), ' points in space,'];
  NOF = [NOF, ' t = ', num2str(Tfinal), '.'];
  plot(x, un, '-b', LW, 3); grid on; zoom on;
  set(gca, FW, Bf, FS, 16)
  xlabel('{\bf  x}', FU, Nor, FS, 0.10)
  ylabel('{\bf  u}', FU, Nor, FS, 0.10, 'rotation', 0, HoA, 'right')
  title(['{\bf ', NOF, '}'], FU, Nor, FS, 0.05)
  umax = max(un); umin = min(un); du = umax-umin;
  axis([0, Length, umin-du/20, umax+du/20])
end
% ============================================================================

function MakeTheSchemeFigure(n, N)
% ============================================================================
SSz = get(0, 'ScreenSize');
SW  = SSz(3);% ------------ Screen Width.
SH  = SSz(4);% ------------ Screen Height.
Sml = min(SW, SH);% ------- Screen minimum length.
% ------------------------- Lengths below in fractions of Sml.
dX = 0.03*n;% --- Figure window distance from screen right edge.
dY = 0.03*n;% --- Figure window distance from screen top   edge.
fW = 6.55*0.07;% --- Figure window width.
fH = 5.00*0.07;% --- Figure window height.
% ------------------------- Figure position.
POF = [SW - (fW + dX)*Sml, SH - (fH + dY)*Sml, fW*Sml, fH*Sml];
%
NOF = ['Solution with N = ', num2str(N), ' points in space.'];
%
figure('Name', NOF, 'NumberTitle', 'off', 'Position', POF)
%
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% EOF